import requests
import urllib.parse
import collections

SEFARIA_BASE_URL = "https://www.sefaria.org/api/"
TEXTS_API_VERSION = "v3/texts/"

class AvailableTextReference:
    def __init__(self, text_ref):
        '''text_ref should be a section e.g. "Pirkei Avot 1"'''
        self.text_ref = text_ref
        url = SEFARIA_BASE_URL + TEXTS_API_VERSION + urllib.parse.quote(text_ref)
        headers = { "accept": "application/json" }
        json = requests.get(url, headers=headers).json()

        if json.get("error") is not None:
            raise KeyError(json["error"])

        language_family_entries = [version["languageFamilyName"]
                                   for version in json["available_versions"]]
        self.language_families = list(set(language_family_entries))

        self.versions = collections.defaultdict(list)
        for version in json["available_versions"]:
            self.versions[version["languageFamilyName"]] += [version["versionTitle"]]

        self.primary_category = json["primary_category"]
        self.index_title = json["indexTitle"]
        self.prev = json["prev"]
        self.title = json["title"]
        self.next = json["next"]
        try:
            self.subsections = len(json["versions"][0]["text"])
        except KeyError as e:
            self.subsections = 0

class TextContent:
    def __init__(self, text_ref, language_family, version_title):
        self.text_ref = text_ref
        self.language_family = language_family
        self.version_title = version_title
        self.version_specifier = "" + language_family + "|" + version_title

        url = SEFARIA_BASE_URL + TEXTS_API_VERSION + \
                urllib.parse.quote(text_ref) + "?" + \
                "version=" + urllib.parse.quote(self.version_specifier)
        headers = { "accept": "application/json" }
        json = requests.get(url, headers=headers).json()

        if json.get("error") is not None:
            raise KeyError(json["error"])

        if json.get("versions") is None or len(json["versions"]) == 0:
            raise ValueError("Response had no versions")

        if json["versions"][0].get("text") is None \
                or len(json["versions"][0]["text"]) == 0:
            raise ValueError("Response version had no text")

        self.text = "".join(json["versions"][0]["text"])
